// ======================================================================
//
// MenuResource.h
// jeff grills
//
// copyright 1997 origin systems inc
//
// ======================================================================

#ifndef MENU_RESOURCE_H
#define MENU_RESOURCE_H

// ======================================================================

class Bitmap;
class Font;
class Iff;
class StringFile;
class MenuElement;

#include "Timer.h"

// ======================================================================

class MenuResource
{
public:

	enum Type
	{
		TypeBitmap,
		TypeFont,
		TypeInteger,
		TypeIntegerAnimation,
		TypeLanguage,
		TypeSetMouse,
		TypeSound
	};

protected:

	MenuResource(void);
	MenuResource(const MenuResource &);
	MenuResource &operator =(const MenuResource &);

protected:

	Type          type;
	char         *id;
	MenuElement  *owner;
	MenuResource *next;

public:

	MenuResource(Type newType, MenuElement *newOwner);
	virtual ~MenuResource(void);

	void          load(Iff *iff);
	void          setNext(MenuResource *newNext);
	void          setOwner(MenuElement *newOwner);

	Type          getType(void) const;
	const char   *getId(void) const;
	MenuResource *getNext(void) const;

	MenuResource *findSharedResource(Type findType, const char *findId);

	virtual void menuEvent(int id, MenuElement *whom);
	virtual void update(void);
};

// --------------------------------------------------------------------

class MenuResourceBitmap : public MenuResource
{
protected:

	Bitmap *bitmap;

protected:

	MenuResourceBitmap(void);
	MenuResourceBitmap(const MenuResourceBitmap &);
	MenuResourceBitmap &operator =(const MenuResourceBitmap &);

public:

	MenuResourceBitmap(MenuElement *newOwner, Iff *iff);
	~MenuResourceBitmap(void);

	Bitmap *getBitmap(void);

	void load(Iff *iff);
};

// --------------------------------------------------------------------

class MenuResourceFont : public MenuResource
{
protected:

	Font *font;
		
protected:

	MenuResourceFont(void);
	MenuResourceFont(const MenuResourceFont &);
	MenuResourceFont &operator =(const MenuResourceFont &);

public:

	MenuResourceFont(MenuElement *newOwner, Iff *iff);
	~MenuResourceFont(void);

	Font *getFont(void);

	void load(Iff *iff);
};

// --------------------------------------------------------------------

class MenuResourceLanguage : public MenuResource
{
protected:

	StringFile *partition;
		
protected:

	MenuResourceLanguage(void);
	MenuResourceLanguage(const MenuResourceLanguage &);
	MenuResourceLanguage &operator =(const MenuResourceLanguage &);

public:

	MenuResourceLanguage(MenuElement *newOwner, Iff *iff);
	~MenuResourceLanguage(void);

	StringFile *getLanguage(void);

	void load(Iff *iff);
};

// --------------------------------------------------------------------

class MenuResourceInteger : public MenuResource
{
private:

	struct ChangeEvent
	{
		int  event;
		int  amount;
		bool absolute;
	};

	struct EventOnValue
	{
		int  value;
		int  event;
	};

private:

	bool         looping;
	int          *changedEvent;
	int          numChangedEvents;
	int          minState;
	int          maxState;
	int          currentState;

	int          changeEvents;
	ChangeEvent  *changeEvent;
	EventOnValue *eventsOnValues;
	int          numEventsOnValues;

private:

	MenuResourceInteger(const MenuResourceInteger &);
	MenuResourceInteger &operator =(const MenuResourceInteger &);

	void         init(void);

public:

	explicit MenuResourceInteger(MenuElement *newOwner=NULL, Iff *iff=NULL);
	virtual ~MenuResourceInteger(void);

	void         load(Iff *iff);

	virtual int  getValue(void) const;
	virtual void setValue(int newValue);

	virtual void menuEvent(int id, MenuElement *whom);
};

// --------------------------------------------------------------------

class MenuResourceIntegerAnimation : public MenuResourceInteger
{
private:

	Timer               timer;
	int                 **animation;
	int                 numAnims;
	int                 currentAnimation;
	int                 currentFrame;

private:

	MenuResourceIntegerAnimation(const MenuResourceIntegerAnimation &);
	MenuResourceIntegerAnimation &operator =(const MenuResourceIntegerAnimation &);

public:

	explicit MenuResourceIntegerAnimation(MenuElement *newOwner=NULL, Iff *iff=NULL);
	virtual ~MenuResourceIntegerAnimation(void);

	void         load(Iff *iff);
	virtual void update(void);
	virtual void setValue(int newValue);
	int          getFrame(void) const;
};

// --------------------------------------------------------------------

class MenuSound : public MenuResource
{
private:

	int             event;
	int             sampleNumber;
#if 0
	FileNamePrefix  path;
#endif
	int             setPriority;
	real            volume;
	real            pan;

private:

	MenuSound(void);
	MenuSound(const MenuSound &);
	MenuSound &operator =(const MenuSound &);

	void         init(void);

public:

	MenuSound(MenuElement *newOwner, Iff *iff);
	virtual ~MenuSound(void);

	void         load(Iff *iff);

	virtual void menuEvent(int id, MenuElement *whom);
};

// --------------------------------------------------------------------

class MenuSetMouse : public MenuResource
{
private:

	struct MousePosition
	{
		int eventId;
		int x;
		int y;
	};

private:

	MousePosition *mousePos;
	int           numMouseEvents;

private:

	MenuSetMouse(void);
	MenuSetMouse(const MenuSetMouse &);
	MenuSetMouse &operator =(const MenuSetMouse &);

	void         init(void);

public:

	MenuSetMouse(MenuElement *newOwner, Iff *iff);
	virtual ~MenuSetMouse(void);

	void         load(Iff *iff);
	virtual void menuEvent(int id, MenuElement *whom);
};

// ======================================================================

inline MenuResource::Type MenuResource::getType(void) const
{
	return type;
}

// ----------------------------------------------------------------------

inline const char *MenuResource::getId(void) const
{
	return id;
}

// ----------------------------------------------------------------------

inline MenuResource *MenuResource::getNext(void) const
{
	return next;
}

// ----------------------------------------------------------------------

inline void MenuResource::setNext(MenuResource *newNext)
{
	next = newNext;
}

// ----------------------------------------------------------------------

inline void MenuResource::setOwner(MenuElement *newOwner)
{
	owner = newOwner;
}

// ======================================================================

inline Bitmap *MenuResourceBitmap::getBitmap(void)
{
	return bitmap;
}

// ======================================================================

inline Font *MenuResourceFont::getFont(void)
{
	return font;
}

// ======================================================================

inline StringFile *MenuResourceLanguage::getLanguage(void)
{
	return partition;
}

// ======================================================================

#endif
